@@ -6,6 +6,8 @@
 #include "StringList.h"
 #include "debug.h"
 
+using namespace std;
+
 StringList::StringList(){
 }
 
@@ -29,12 +31,12 @@ StringList::~StringList(){
 }
 
 char** StringList::toCharArray() const{
-    if(size() > 0){
-        char** ret = new char*[size()+1];
+    if(!empty()){
+        char** ret = (char**) malloc(sizeof(char*) * (size()+1));
         StringList::const_iterator i;
         int j=0;
         for(i=begin(); i != end(); i++,j++){
-            ret[j]=new char[i->size()+1];
+            ret[j]=(char*) malloc(sizeof(char) * (i->size()+1));
             i->copy(ret[j],string::npos);
             ret[j][i->size()]=0;
         }
@@ -53,6 +55,10 @@ size_t StringList::size() const{
     return m_data.size();
 }
 
+bool StringList::empty() const{
+    return m_data.empty();
+}
+
 StringList::const_iterator StringList::begin() const{
     return m_data.begin();
 }
