@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/compare.c,v 1.75.2.5 2002/10/10 01:45:55 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/compare.c,v 1.75.2.8 2003/02/10 19:22:46 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /*
@@ -22,6 +22,9 @@
 
 #include "ldap_pvt.h"
 #include "slap.h"
+#ifdef LDAP_SLAPI
+#include "slapi.h"
+#endif
 
 static int compare_entry(
 	Connection *conn,
@@ -42,12 +45,16 @@ do_compare(
 	struct berval ndn = { 0, NULL };
 	struct berval desc = { 0, NULL };
 	struct berval value = { 0, NULL };
-	AttributeAssertion ava = { 0 };
+	AttributeAssertion ava = { NULL, { 0, NULL } };
 	Backend	*be;
 	int rc = LDAP_SUCCESS;
 	const char *text = NULL;
 	int manageDSAit;
 
+#ifdef LDAP_SLAPI
+	Slapi_PBlock *pb = op->o_pb;
+#endif
+
 	ava.aa_desc = NULL;
 
 #ifdef NEW_LOGGING
@@ -264,8 +271,33 @@ do_compare(
 		ava.aa_desc->ad_cname.bv_val, 0 );
 
 
-	/* deref suffix alias if appropriate */
-	suffix_alias( be, &ndn );
+#if defined( LDAP_SLAPI )
+	slapi_x_backend_set_pb( pb, be );
+	slapi_x_connection_set_pb( pb, conn );
+	slapi_x_operation_set_pb( pb, op );
+	slapi_pblock_set( pb, SLAPI_COMPARE_TARGET, (void *)dn.bv_val );
+	slapi_pblock_set( pb, SLAPI_MANAGEDSAIT, (void *)manageDSAit );
+	slapi_pblock_set( pb, SLAPI_COMPARE_TYPE, (void *)desc.bv_val );
+	slapi_pblock_set( pb, SLAPI_COMPARE_VALUE, (void *)&value );
+
+	rc = doPluginFNs( be, SLAPI_PLUGIN_PRE_COMPARE_FN, pb );
+	if ( rc != 0 ) {
+		/*
+		 * A preoperation plugin failure will abort the
+		 * entire operation.
+		 */
+#ifdef NEW_LOGGING
+		LDAP_LOG( OPERATION, INFO, "do_compare: compare preoperation plugin "
+				"failed\n", 0, 0, 0);
+#else
+		Debug(LDAP_DEBUG_TRACE, "do_compare: compare preoperation plugin "
+				"failed.\n", 0, 0, 0);
+#endif
+		if ( slapi_pblock_get( pb, SLAPI_RESULT_CODE, (void *)&rc ) != 0)
+			rc = LDAP_OTHER;
+		goto cleanup;
+	}
+#endif /* defined( LDAP_SLAPI ) */
 
 	if ( be->be_compare ) {
 		(*be->be_compare)( be, conn, op, &pdn, &ndn, &ava );
@@ -275,6 +307,18 @@ do_compare(
 			NULL, NULL );
 	}
 
+#if defined( LDAP_SLAPI )
+	if ( doPluginFNs( be, SLAPI_PLUGIN_POST_COMPARE_FN, pb ) != 0 ) {
+#ifdef NEW_LOGGING
+		LDAP_LOG( OPERATION, INFO, "do_compare: compare postoperation plugins "
+				"failed\n", 0, 0, 0 );
+#else
+		Debug(LDAP_DEBUG_TRACE, "do_compare: compare postoperation plugins "
+				"failed.\n", 0, 0, 0);
+#endif
+	}
+#endif /* defined( LDAP_SLAPI ) */
+
 cleanup:
 	free( pdn.bv_val );
 	free( ndn.bv_val );
