@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/modify.c,v 1.127.2.14 2003/03/12 20:58:48 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/modify.c,v 1.127.2.16 2004/01/24 01:21:41 hyc Exp $ */
 /*
  * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
@@ -309,11 +309,17 @@ do_modify(
 	if ( (be = select_backend( &ndn, manageDSAit, 0 )) == NULL ) {
 		BerVarray ref = referral_rewrite( default_referral,
 			NULL, &pdn, LDAP_SCOPE_DEFAULT );
+		if ( ref == NULL ) ref = default_referral;
+		if ( ref != NULL ) {
+			send_ldap_result( conn, op, rc = LDAP_REFERRAL,
+				NULL, NULL, ref, NULL );
 
-		send_ldap_result( conn, op, rc = LDAP_REFERRAL,
-			NULL, NULL, ref ? ref : default_referral, NULL );
-
-		ber_bvarray_free( ref );
+			if ( ref != default_referral ) ber_bvarray_free( ref );
+		} else {
+			send_ldap_result( conn, op,
+					rc = LDAP_UNWILLING_TO_PERFORM,
+					NULL, "referral missing", NULL, NULL );
+		}
 		goto cleanup;
 	}
 
@@ -393,6 +399,8 @@ do_modify(
 			const char *text;
 			char textbuf[SLAP_TEXT_BUFLEN];
 			size_t textlen = sizeof textbuf;
+			slap_replog_ctx ctx;
+			slap_callback cb = {0};
 
 			rc = slap_mods_check( modlist, update, &text,
 				textbuf, textlen );
@@ -421,27 +429,41 @@ do_modify(
 				}
 			}
 
-			if ( (*be->be_modify)( be, conn, op, &pdn, &ndn, modlist ) == 0
 #ifdef SLAPD_MULTIMASTER
-				&& !repl_user
+			if ( !repl_user )
 #endif
-			) {
-				/* but we log only the ones not from a replicator user */
-				replog( be, op, &pdn, &ndn, modlist );
+			{
+				ctx.prev = op->o_callback;
+				ctx.be = be;
+				ctx.dn = &pdn;
+				ctx.ndn = &ndn;
+				ctx.change = modlist;
+				cb.sc_private = &ctx;
+				cb.sc_response = slap_replog_cb;
+				op->o_callback = &cb;
 			}
+			rc = (*be->be_modify)( be, conn, op, &pdn, &ndn, modlist );
 
 #ifndef SLAPD_MULTIMASTER
 		/* send a referral */
 		} else {
 			BerVarray defref = be->be_update_refs
 				? be->be_update_refs : default_referral;
-			BerVarray ref = referral_rewrite( defref,
-				NULL, &pdn, LDAP_SCOPE_DEFAULT );
+			if ( defref != NULL ) {
+				BerVarray ref = referral_rewrite( defref,
+					NULL, &pdn, LDAP_SCOPE_DEFAULT );
 
-			send_ldap_result( conn, op, rc = LDAP_REFERRAL, NULL, NULL,
-				ref ? ref : defref, NULL );
+				send_ldap_result( conn, op, rc = LDAP_REFERRAL,
+						NULL, NULL,
+						ref ? ref : defref, NULL );
 
-			ber_bvarray_free( ref );
+				ber_bvarray_free( ref );
+			} else {
+				send_ldap_result( conn, op,
+						rc = LDAP_UNWILLING_TO_PERFORM,
+						NULL, "referral missing",
+						NULL, NULL );
+			}
 #endif
 		}
 	} else {
